' *********************************************************
' **  ExpressPlayer for Roku
' **  Copyright (c) 2014 Intertrust
' ** 
' **  Includes code from Simple Grid Screen Demonstration App
' **  Copyright (c) 2010 Roku Inc. All Rights Reserved.
' *********************************************************


'************************************************************
'** Application startup
'************************************************************

Sub Main()
    'initialize theme attributes like titles, logos and overhang color
    initTheme()
    defaultListUrl = "http://cdn.expressplay.com/json/ExpressPlayerRoku.json"
    'defaultListUrl = "http://expressplay.s3.amazonaws.com/json/ExpressPlayerRoku.json"
    url = showInputScreen(defaultListUrl)
End Sub


'*************************************************************
'** Set the configurable theme attributes for the application
'** 
'** Configure the custom overhang and Logo attributes
'** These attributes affect the branding of the application
'** and are artwork, colors and offsets specific to the app
'*************************************************************

Sub initTheme()
    app = CreateObject("roAppManager")
    app.SetTheme(CreateDefaultTheme())
End Sub


'******************************************************
'** @return The default application theme.
'** Screens can make slight adjustments to the default
'** theme by getting it from here and then overriding
'** individual theme attributes.
'******************************************************

Function CreateDefaultTheme() as Object
    theme = CreateObject("roAssociativeArray")

    theme.ThemeType = "generic-dark"

    ' All these are greyscales
    theme.GridScreenBackgroundColor = "#363636"
    theme.GridScreenMessageColor    = "#FFFFFF"
    theme.GridScreenRetrievingColor = "#CCCCCC"
    theme.GridScreenListNameColor   = "#FFFFFF"

    ' Color values work here
    theme.GridScreenDescriptionTitleColor    = "#363636"
    theme.GridScreenDescriptionDateColor     = "#363636"
    theme.GridScreenDescriptionRuntimeColor  = "#363636"
    theme.GridScreenDescriptionSynopsisColor = "#363636"
    
    'used in the Grid Screen
    theme.CounterTextLeft           = "#FFFFFF"
    theme.CounterSeparator          = "#FFFFFF"
    theme.CounterTextRight          = "#FFFFFF"
    
    theme.GridScreenLogoHD          = "pkg:/images/Overhang_Test_HD.png"

    theme.OverhangSliceHD           = "pkg:/images/Overhang_BackgroundSlice_HD.png"
    theme.OverhangSliceSD           = "pkg:/images/Overhang_BackgroundSlice_SD43.png"
    theme.OverhangPrimaryLogoHD     = "pkg:/images/Logo_Overhang_HD.png"
    theme.OverhangPrimaryLogoSD     = "pkg:/images/Logo_Overhang_SD43.png"
    theme.OverhangPrimaryLogoOffsetHD_X   = "100"
    theme.OverhangPrimaryLogoOffsetHD_Y   = "43"
    theme.OverhangPrimaryLogoOffsetSD_X   = "62"
    theme.OverhangPrimaryLogoOffsetSD_Y   = "33"

    theme.GridScreenLogoOffsetHD_X  = "0"
    theme.GridScreenLogoOffsetHD_Y  = "0"
    theme.GridScreenOverhangHeightHD = "99"

    theme.GridScreenLogoSD          = "pkg:/images/Overhang_Test_SD43.png"
    theme.GridScreenOverhangHeightSD = "66"
    theme.GridScreenLogoOffsetSD_X  = "0"
    theme.GridScreenLogoOffsetSD_Y  = "0"
    
    ' to use your own focus ring artwork 
    'theme.GridScreenFocusBorderSD        = "pkg:/images/GridCenter_Border_Movies_SD43.png"
    'theme.GridScreenBorderOffsetSD  = "(-26,-25)"
    'theme.GridScreenFocusBorderHD        = "pkg:/images/GridCenter_Border_Movies_HD.png"
    'theme.GridScreenBorderOffsetHD  = "(-28,-20)"
    
    ' to use your own description background artwork
    'theme.GridScreenDescriptionImageSD  = "pkg:/images/Grid_Description_Background_SD43.png"
    'theme.GridScreenDescriptionOffsetSD = "(125,170)"
    'theme.GridScreenDescriptionImageHD  = "pkg:/images/Grid_Description_Background_HD.png"
    'theme.GridScreenDescriptionOffsetHD = "(190,255)"
    return theme
End Function


'******************************************************
'** Display the gird screen and wait for events from 
'** the screen. The screen will show retreiving while
'** we fetch and parse the feeds for the show posters
'******************************************************

Function showGridScreen(url as string)
    port = CreateObject("roMessagePort") 
    screen = CreateObject("roGridScreen")
    screen.SetMessagePort(port)
    screen.SetDisplayMode("scale-to-fill")
    screen.SetGridStyle("Flat-Movie")
    print "enter showGridScreen"
    screen.setupLists(1)
    screen.SetListNames([url])
    playlist = getPlaylist(url)
    screen.SetContentList(0, playlist)
    screen.Show()

    while true
        print "Waiting for message"
        msg = wait(0, screen.GetMessagePort())
        print "Got Message: ";type(msg)
        if type(msg) = "roGridScreenEvent" then
            print "msg= "; msg.GetMessage() " , index= "; msg.GetIndex(); " data= "; msg.getData()
            if msg.isListItemFocused() then
                print "list item focused | current show = "; msg.GetIndex()
            else if msg.isListItemSelected() then
                row = msg.GetIndex()
                selection = msg.getData()
                print "list item selected row= "; row; " selection= "; selection
                if playlist[selection].url <> invalid then
                    video = CreateObject("roAssociativeArray")
                    video.url = playlist[selection].url
                    if playlist[selection].container <> invalid 
                        if (playlist[selection].container = "hls" or playlist[selection].container = "HLS") then
                            video.StreamFormat = "hls"
                        else if (playlist[selection].container = "ism" or playlist[selection].container = "ISM") then
                            video.StreamFormat = "ism"
                        end if
                    end if
                    m.curShow = displayVideo(video)
                end if
            else if msg.isScreenClosed() then
                print "closing"
                return null
            end if
        end If
    end while
End Function


'********************************************************************
'** Return an array 
'** of ContentMetaData objects (roAssociativeArray's) representing 
'** the shows for the category. For this example, we just cheat and
'** create and return a static array with just the minimal items
'** set, but ideally, you'd go to a feed service, fetch and parse
'** this data dynamically, so content for each category is dynamic
'********************************************************************

Function getPlaylist(playlistUrl) As Object
    print "getting shows for url "; playlistUrl

    jsonRequest = CreateObject("roUrlTransfer")
    jsonRequest.SetURL(playlistUrl)
    print "got the playlist"
    showListJson = ParseJson(jsonRequest.GetToString())
    showList = []
    i = 0
    posterHD = "pkg:/images/Poster_Movies_HD.png"
    posterSD = "pkg:/images/Poster_Movies_SD43.png"
    if showListJson <> invalid
        For Each item In showListJson
            if item.HDPosterUrl <> invalid
                posterHD = item.HDPosterUrl
            end if 
            if item.SDPosterUrl <> invalid
                posterSD = item.SDPosterUrl
            end if
            showList[i] = {
                Title:          item.title,
                releaseDate:    item.releaseDate,
                rating:         "PG",
                Description:    item.description,
                numEpisodes:    1,
                contentType:    "movie",
                HDPosterUrl:    posterHD,
                SDPosterUrl:    posterSD,
                StarRating:     80,
                UserStarRating: 40,
                url:            item.url,
                container:      item.container
            }
            i = i + 1
        End For
    end if
    return showList
End Function


Function showInputScreen(url as string) As string
     screen = CreateObject("roKeyboardScreen")
     port = CreateObject("roMessagePort") 
     screen.SetMessagePort(port)
     screen.SetTitle("Content List URL")
     screen.SetText(url)
     screen.SetDisplayText("Enter the URL to a JSON formatted list.")
     screen.SetMaxLength(8)
     screen.AddButton(1, "Load This Content List")
     screen.Show()
     while true 
         msg = wait(0, screen.GetMessagePort())
         print "message received"
         if type(msg) = "roKeyboardScreenEvent" then
             if msg.isScreenClosed() then
                print "closing"
                 return ""
             else if msg.isButtonPressed() then
                 print "Evt:"; msg.GetMessage ();" idx:"; msg.GetIndex()
                 if msg.GetIndex() = 1 then
                     url = screen.GetText()
                     print "URL: "; url
                     showGridScreen(url)
                     'return url
                 end if
             end if
         end if
     end while 
End Function
